<?php

namespace Growmatik\Controller;

use Growmatik\Event_Transport;

class Redirect_Controller {
	const OVERRIDE_REFERRER_COOKIE_NAME = '_growmatik_override_referrer';

	const VALID_SCHEMES = array( 'http', 'https', 'ftp' );

	public function __construct() {
		add_action( 'template_redirect', array( $this, 'growmatik_template_redirect' ) );
	}

	public static function add_rewrite_rules() {
		add_rewrite_rule(
			'^gmredirect/([^/]*)/(.*)$',
			'index.php?pagename=gmredirect&gmr_type=$matches[1]&gmr_url=$matches[2]',
			'top'
		);
	}

	public static function register_redirect_query_vars( $vars ) {
		array_push(
			$vars,
			'gmr_url',
			'gmr_type',
			'gmr_oid',
			'gmr_oty',
			'gmr_sid',
			'gmr_rid',
			'gmr_raid',
			'gmr_rtid',
			'gmr_hid',
			'gmr_ref',
			'gmr_uid'
		);
		return $vars;
	}

	public function growmatik_template_redirect() {
		$url  = rawurldecode( rawurldecode( get_query_var( 'gmr_url' ) ) );
		$type = get_query_var( 'gmr_type' );

		if ( $url && $type ) {
			if ( ! in_array( strtok( $url, ':' ), self::VALID_SCHEMES, true ) ) {
				return;
			}
			if ( (string) get_query_var( 'gmr_sid' ) !== (string) get_option( 'growmatik_site_id' ) ) {
				return; // Using another site id, must not work on this site
			}

			if ( '/' === $url[0] ) {
				$url         = home_url( $url );
				$is_internal = true;
			} else {
				$own_host    = wp_parse_url( get_home_url(), PHP_URL_HOST );
				$target_host = wp_parse_url( $url, PHP_URL_HOST );

				// loose comparison, ignoring scheme and port.
				$is_internal = ( $own_host === $target_host );
			}

			if ( $is_internal ) {
				// Set only for internal URLs to make sure the cookie is instantly picked up and removed by front-end.
				// If we set it for URLs heading out of the site, it will remain there on next visit to the blog and
				// will send an invalid referrer on next page load.
				$this->set_override_referrer_cookie( $type );
			} else {
				// Delete the previous cookie, if any; to not break things in case of chained internal url redirects
				$this->remove_override_referrer_cookie();

				$this->send_redirect_event( $type, $url );
			}

			wp_redirect( $url ); // phpcs:ignore
			exit();
		}
	}

	private function get_redirect_data( $type, $url = null ) {
		$redirect_data = array(
			'type' => $type,
			'oid'  => get_query_var( 'gmr_oid', null ),
			'oty'  => get_query_var( 'gmr_oty', null ),
			'sid'  => get_query_var( 'gmr_sid', null ),
			'rid'  => get_query_var( 'gmr_rid', null ),
			'raid' => get_query_var( 'gmr_raid', null ),
			'rtid' => get_query_var( 'gmr_rtid', null ),
			'hid'  => get_query_var( 'gmr_hid', null ),
			'ref'  => get_query_var( 'gmr_ref', null ),
			'uid'  => get_query_var( 'gmr_uid', null ),
		);

		if ( isset( $url ) ) {
			$redirect_data['url'] = $url;
		}

		return $redirect_data;
	}

	private function set_override_referrer_cookie( $type ) {
		$redirect_data = wp_json_encode( $this->get_redirect_data( $type ) );
		setcookie( self::OVERRIDE_REFERRER_COOKIE_NAME, $redirect_data, 0, '/' );
	}

	private function remove_override_referrer_cookie() {
		setcookie( self::OVERRIDE_REFERRER_COOKIE_NAME, '', time() - 3600, '/' );
	}

	private function send_redirect_event( $type, $url ) {
		$event_name = 'redirect:' . $type;
		$event_data = array(
			'redirect' => $this->get_redirect_data( $type, $url ),
		);

		// phpcs:ignore WordPress.WP.CapitalPDangit.Misspelled
		$transport = new Event_Transport( 'wordpress' );
		$transport->send_event( $event_name, $event_data );
	}
}
