<?php

namespace Growmatik\EDD;

use EDD_Discount;
use WP_Error;
use WP_REST_Response;
use function Growmatik\meta_site_prefix;

class Discounts {

	public function index( $request ) {

		$discount_id = $request->get_param( 'id' );
		return $this->get_discount( $discount_id );
	}

	/** @return EDD_Discount|WP_Error */
	private function retrieve_discount( $id ) {
		$discount = edd_get_discount( $id );

		if ( ! ( $discount instanceof EDD_Discount ) ) {
			return new WP_Error( 'invalid', 'invalid_coupon', array( 'status' => 404 ) );
		}
		return $discount;
	}

	/** @return WP_Error|WP_REST_Response */
	private function get_discount( $id ) {
		$gm_prefix     = meta_site_prefix();
		$entity_prefix = "{$gm_prefix}_entity";
		$discount      = $this->retrieve_discount( $id );

		if ( ! ( $discount instanceof EDD_Discount ) ) {
			return $discount;
		}

		$discount_data = array(
			'id'            => $discount->id,
			'status'        => $discount->status,
			'code'          => $discount->code,
			'discount_type' => $discount->type,
			'coupon_amount' => $discount->amount,
			'product_ids'   => $discount->product_reqs,
			'entity_type'   => $discount->get_meta( "{$entity_prefix}_type" ),
			'entity_id'     => $discount->get_meta( "{$entity_prefix}_id" ),
		);

		$data = array( 'coupon' => $discount_data );

		$response = new WP_REST_Response( $data );

		$response->set_status( 200 );

		return $response;

	}

	/** @return WP_Error|WP_REST_Response */
	public function add( $request ) {
		$args                 = array();
		$existing_discount_id = null;

		$gm_prefix     = meta_site_prefix();
		$entity_prefix = "{$gm_prefix}_entity";

		if ( $request->get_param( 'id' ) ) {
			$existing_discount_id = $request->get_param( 'id' );
			$args['code']         = edd_get_discount( $existing_discount_id )->code;
		} else {
			$args['code'] = strtoupper( substr( md5( wp_rand() ), 0, 12 ) );
		}

		$args['scope']  = 'global';
		$args['status'] = 'active';

		$args['amount']       = $request->get_param( 'coupon_amount' );
		$args['amount_type']  = $this->validate_discount_type( $request->get_param( 'discount_type' ) );
		$args['product_reqs'] = $request->get_param( 'product_ids' );

		// Only assign a name on coupon create; tenant may change the coupon name later so don't touch it on updates.
		if ( ! $existing_discount_id ) {
			$args['name']    = '(Growmatik) ' . $args['code'];
			$new_discount_id = edd_add_discount( $args );

			edd_update_adjustment_meta( $new_discount_id, "{$entity_prefix}_type", $request->get_param( 'entity_type' ) );
			edd_update_adjustment_meta( $new_discount_id, "{$entity_prefix}_id", $request->get_param( 'entity_id' ) );
			edd_update_adjustment_meta( $new_discount_id, "{$gm_prefix}_discount", 1 );
		} else {
			$new_discount_id = $existing_discount_id;

			edd_update_discount( $existing_discount_id, $args );
		}

		return $this->get_discount( $new_discount_id );
	}

	/** @return string */
	private function validate_discount_type( $discount_type ) {
		$valid_discount_types = array( 'percent', 'flat' );

		if ( ! in_array( $discount_type, $valid_discount_types, true ) ) {
			$discount_type = 'percent';
		}

		return $discount_type;
	}

	/** @return WP_Error|WP_REST_Response */
	public function remove( $request ) {
		$discount_id = $request->get_param( 'id' );

		$discount = $this->retrieve_discount( $discount_id );

		if ( ! ( $discount instanceof EDD_Discount ) ) {
			return $discount;
		}

		$response = $this->get_discount( $discount_id );

		edd_delete_discount( $discount_id );

		return $response;
	}

	public static function remove_data() {
		if ( ! is_edd_active() ) {
			return;
		}

		global $wpdb;
		$gm_prefix = meta_site_prefix();

		// phpcs:ignore WordPress.DB.PreparedSQL.InterpolatedNotPrepared
		$wpdb->query( "DELETE FROM $wpdb->edd_adjustmentmeta WHERE meta_key LIKE '$gm_prefix%'" );
	}

}
