<?php

namespace Growmatik\Form_Integration\WPForms;

use Growmatik\Form_Integration\Integration;

class Form {

	public function __construct() {
		add_action( 'wpforms_process_complete', array( $this, 'after_submission' ), 10, 4 );
		add_action( 'wpforms_field_options_bottom_basic-options', array( $this, 'add_settings' ), 10, 2 );
	}

	public function get_growmatik_fields() {
		$fields = array();

		$fields['none'] = __( '-None-', 'growmatik' );

		$all_attributes = Integration::instance()->get_all_attributes();

		foreach ( $all_attributes as $field_key => $label ) {
			$fields[ $field_key ] = $label;
		}

		return $fields;
	}

	public function add_settings( $field, $instance ) {

		$exluded_field_types = array(
			'divider',
			'file-upload',
			'password',
			'pagebreak',
			'captcha',
		);

		if ( empty( $field['type'] ) || in_array( $field['type'], $exluded_field_types, true ) ) {
			return;
		}

		// Field label.
		$growmatik_field_label = $instance->field_element(
			'label',
			$field,
			array(
				'slug'    => 'growmatik',
				'value'   => esc_html__( 'Map to a Growmatik field', 'growmatik' ),
				'tooltip' => esc_html__( 'Map this field to a Growmatik field.', 'growmatik' ),
			),
			false
		);

		// Field Select box.
		$growmatik_field_setting = $instance->field_element(
			'select',
			$field,
			array(
				'slug'    => 'growmatik',
				'value'   => ! empty( $field['growmatik'] ) ? esc_attr( $field['growmatik'] ) : 'none',
				'desc'    => esc_html__( 'Map to a Growmatik field', 'growmatik' ),
				'options' => $this->get_growmatik_fields(),
			),
			false
		);
		$args                    = array(
			'slug'    => 'growmatik',
			'content' => $growmatik_field_label . $growmatik_field_setting,
		);
		$instance->field_element( 'row', $field, $args );

	}

	public function after_submission( array $fields, array $entry, array $form, int $entry_id ) {

		$form_data = array();

		$form_data['siteId']   = get_option( 'growmatik_site_id' );
		$form_data['formName'] = $form['settings']['form_title'];
		$form_data['tags']     = array();
		$form_data['formData'] = array();
		$vals                  = array( 'on', true, 'true', 'checked', '1', 1, 'granted' );

		foreach ( $form['fields'] as $field ) {
			if ( isset( $field['growmatik'] ) && 'none' !== $field['growmatik'] ) {
				$growmatik_field = str_replace( 'growmatik-', '', $field['growmatik'] );
				if ( in_array( $growmatik_field, array( '012', '013' ), true ) ) {
					$fields[ $field['id'] ]['value'] = in_array( $fields[ $field['id'] ]['value'], $vals, true )
					? 'granted' : 'denied';
				}
				$form_data['formData'][] = array(
					'key'   => $growmatik_field,
					'value' => $fields[ $field['id'] ]['value'],
					'type'  => Integration::instance()->get_attribute_type( $growmatik_field ),
				);
			}
		}

		$response = Integration::instance()->form_submission( $form_data );

		if ( is_wp_error( $response ) ) {
			error_log( __( 'Growmatik: Internal server error.', 'growmatik' ) ); // phpcs:ignore
		} elseif ( isset( $response['errors'] ) && isset( $response['error']['api_error'] ) ) {
			error_log( (string) $response['error']['api_error'][0] ); // phpcs:ignore
		}
	}
}
